@@ -27,6 +27,7 @@
 
 #include <JavaScriptCore/runtime.h>
 #include <JavaScriptCore/object.h>
+#include <JavaScriptCore/protect.h>
 
 namespace KJS {
 
@@ -47,7 +48,7 @@ public:
 
     virtual bool canPut(ExecState *exec, const Identifier &propertyName) const;
 
-    virtual bool hasProperty(ExecState *exec,
+    virtual bool hasOwnProperty(ExecState *exec,
 			     const Identifier &propertyName) const;
 
 
@@ -59,9 +60,12 @@ public:
     void setInternalInstance (Bindings::Instance *i) { instance = i; }
     Bindings::Instance *getInternalInstance() const { return instance; }
 
-private:
+    virtual bool implementsCall() const;
+    virtual Value call(ExecState *exec, Object &thisObj, const List &args);
     
     static const ClassInfo info;
+
+private:
     Bindings::Instance *instance;
     bool ownsInstance;
 };
