@@ -31,6 +31,7 @@
 #include "operations.h"
 #include "debugger.h"
 #include "context.h"
+#include "shared_ptr.h"
 
 #include <stdio.h>
 #include <errno.h>
@@ -38,13 +39,18 @@
 #include <assert.h>
 #include <string.h>
 
-using namespace KJS;
+#if APPLE_CHANGES
+#include <unicode/uchar.h>
+#endif
+
+using namespace kxmlcore;
+
+namespace KJS {
 
 // ----------------------------- FunctionImp ----------------------------------
 
 const ClassInfo FunctionImp::info = {"Function", &InternalFunctionImp::info, 0, 0};
 
-namespace KJS {
   class Parameter {
   public:
     Parameter(const Identifier &n) : name(n), next(0L) { }
@@ -52,11 +58,10 @@ namespace KJS {
     Identifier name;
     Parameter *next;
   };
-};
 
 FunctionImp::FunctionImp(ExecState *exec, const Identifier &n)
   : InternalFunctionImp(
-      static_cast<FunctionPrototypeImp*>(exec->interpreter()->builtinFunctionPrototype().imp())
+      static_cast<FunctionPrototypeImp*>(exec->lexicalInterpreter()->builtinFunctionPrototype().imp())
       ), param(0L), ident(n)
 {
 }
@@ -73,9 +78,20 @@ bool FunctionImp::implementsCall() const
 
 Value FunctionImp::call(ExecState *exec, Object &thisObj, const List &args)
 {
-  Object &globalObj = exec->interpreter()->globalObject();
+  Object &globalObj = exec->dynamicInterpreter()->globalObject();
 
-  Debugger *dbg = exec->interpreter()->imp()->debugger();
+  // enter a new execution context
+  ContextImp ctx(globalObj, exec->dynamicInterpreter()->imp(), thisObj, codeType(),
+                 exec->context().imp(), this, &args);
+  ExecState newExec(exec->dynamicInterpreter(), &ctx);
+  newExec.setException(exec->exception()); // could be null
+
+  // assign user supplied arguments to parameters
+  processParameters(&newExec, args);
+  // add variable declarations (initialized to undefined)
+  processVarDecls(&newExec);
+
+  Debugger *dbg = exec->dynamicInterpreter()->imp()->debugger();
   int sid = -1;
   int lineno = -1;
   if (dbg) {
@@ -85,29 +101,18 @@ Value FunctionImp::call(ExecState *exec, Object &thisObj, const List &args)
     }
 
     Object func(this);
-    bool cont = dbg->callEvent(exec,sid,lineno,func,args);
+    bool cont = dbg->callEvent(&newExec,sid,lineno,func,args);
     if (!cont) {
       dbg->imp()->abort();
       return Undefined();
     }
   }
 
-  // enter a new execution context
-  ContextImp ctx(globalObj, exec->interpreter()->imp(), thisObj, codeType(),
-                 exec->context().imp(), this, &args);
-  ExecState newExec(exec->interpreter(), &ctx);
-  newExec.setException(exec->exception()); // could be null
-
-  // assign user supplied arguments to parameters
-  processParameters(&newExec, args);
-  // add variable declarations (initialized to undefined)
-  processVarDecls(&newExec);
-
   Completion comp = execute(&newExec);
 
   // if an exception occured, propogate it back to the previous execution object
   if (newExec.hadException())
-    exec->setException(newExec.exception());
+    comp = Completion(Throw, newExec.exception());
 
 #ifdef KJS_VERBOSE
   if (comp.complType() == Throw)
@@ -119,8 +124,14 @@ Value FunctionImp::call(ExecState *exec, Object &thisObj, const List &args)
 #endif
 
   if (dbg) {
+    if (inherits(&DeclaredFunctionImp::info))
+      lineno = static_cast<DeclaredFunctionImp*>(this)->body->lastLine();
+
+    if (comp.complType() == Throw)
+        newExec.setException(comp.value());
+
     Object func(this);
-    int cont = dbg->returnEvent(exec,sid,lineno,func);
+    int cont = dbg->returnEvent(&newExec,sid,lineno,func);
     if (!cont) {
       dbg->imp()->abort();
       return Undefined();
@@ -211,7 +222,7 @@ Value FunctionImp::get(ExecState *exec, const Identifier &propertyName) const
                     (context->activationObject())->get(exec, propertyName);
             context = context->callingContext();
         }
-        return Undefined();
+        return Null();
     }
     
     // Compute length of parameters.
@@ -235,11 +246,11 @@ void FunctionImp::put(ExecState *exec, const Identifier &propertyName, const Val
     InternalFunctionImp::put(exec, propertyName, value, attr);
 }
 
-bool FunctionImp::hasProperty(ExecState *exec, const Identifier &propertyName) const
+bool FunctionImp::hasOwnProperty(ExecState *exec, const Identifier &propertyName) const
 {
     if (propertyName == argumentsPropertyName || propertyName == lengthPropertyName)
         return true;
-    return InternalFunctionImp::hasProperty(exec, propertyName);
+    return InternalFunctionImp::hasOwnProperty(exec, propertyName);
 }
 
 bool FunctionImp::deleteProperty(ExecState *exec, const Identifier &propertyName)
@@ -259,16 +270,9 @@ DeclaredFunctionImp::DeclaredFunctionImp(ExecState *exec, const Identifier &n,
   : FunctionImp(exec,n), body(b)
 {
   Value protect(this);
-  body->ref();
   setScope(sc);
 }
 
-DeclaredFunctionImp::~DeclaredFunctionImp()
-{
-  if ( body->deref() )
-    delete body;
-}
-
 bool DeclaredFunctionImp::implementsConstruct() const
 {
   return true;
@@ -282,11 +286,11 @@ Object DeclaredFunctionImp::construct(ExecState *exec, const List &args)
   if (p.type() == ObjectType)
     proto = Object(static_cast<ObjectImp*>(p.imp()));
   else
-    proto = exec->interpreter()->builtinObjectPrototype();
+    proto = exec->lexicalInterpreter()->builtinObjectPrototype();
 
   Object obj(new ObjectImp(proto));
 
-  Value res = call(exec,obj,args);
+  Value res = Object(this).call(exec,obj,args);
 
   if (res.type() == ObjectType)
     return Object::dynamicCast(res);
@@ -314,14 +318,14 @@ const ClassInfo ArgumentsImp::info = {"Arguments", 0, 0, 0};
 
 // ECMA 10.1.8
 ArgumentsImp::ArgumentsImp(ExecState *exec, FunctionImp *func)
-  : ArrayInstanceImp(exec->interpreter()->builtinObjectPrototype().imp(), 0)
+  : ArrayInstanceImp(exec->lexicalInterpreter()->builtinObjectPrototype().imp(), 0)
 {
   Value protect(this);
   putDirect(calleePropertyName, func, DontEnum);
 }
 
 ArgumentsImp::ArgumentsImp(ExecState *exec, FunctionImp *func, const List &args)
-  : ArrayInstanceImp(exec->interpreter()->builtinObjectPrototype().imp(), args)
+  : ArrayInstanceImp(exec->lexicalInterpreter()->builtinObjectPrototype().imp(), args)
 {
   Value protect(this);
   putDirect(calleePropertyName, func, DontEnum);
@@ -358,11 +362,11 @@ void ActivationImp::put(ExecState *exec, const Identifier &propertyName, const V
     ObjectImp::put(exec, propertyName, value, attr);
 }
 
-bool ActivationImp::hasProperty(ExecState *exec, const Identifier &propertyName) const
+bool ActivationImp::hasOwnProperty(ExecState *exec, const Identifier &propertyName) const
 {
     if (propertyName == argumentsPropertyName)
         return true;
-    return ObjectImp::hasProperty(exec, propertyName);
+    return ObjectImp::hasOwnProperty(exec, propertyName);
 }
 
 bool ActivationImp::deleteProperty(ExecState *exec, const Identifier &propertyName)
@@ -493,6 +497,115 @@ static Value decode(ExecState *exec, const List &args, const char *do_not_unesca
   return String(s);
 }
 
+static bool isStrWhiteSpace(unsigned short c)
+{
+    switch (c) {
+        case 0x0009:
+        case 0x000A:
+        case 0x000B:
+        case 0x000C:
+        case 0x000D:
+        case 0x0020:
+        case 0x00A0:
+        case 0x2028:
+        case 0x2029:
+            return true;
+        default:
+#if APPLE_CHANGES
+            return u_charType(c) == U_SPACE_SEPARATOR;
+#else
+            // ### properly support other Unicode Zs characters
+            return false;
+#endif
+    }
+}
+
+static int parseDigit(unsigned short c, int radix)
+{
+    int digit = -1;
+
+    if (c >= '0' && c <= '9') {
+        digit = c - '0';
+    } else if (c >= 'A' && c <= 'Z') {
+        digit = c - 'A' + 10;
+    } else if (c >= 'a' && c <= 'z') {
+        digit = c - 'a' + 10;
+    }
+
+    if (digit >= radix)
+        return -1;
+    return digit;
+}
+
+static double parseInt(const UString &s, int radix)
+{
+    int length = s.size();
+    int p = 0;
+
+    while (p < length && isStrWhiteSpace(s[p].uc)) {
+        ++p;
+    }
+
+    double sign = 1;
+    if (p < length) {
+        if (s[p] == '+') {
+            ++p;
+        } else if (s[p] == '-') {
+            sign = -1;
+            ++p;
+        }
+    }
+
+    if ((radix == 0 || radix == 16) && length - p >= 2 && s[p] == '0' && (s[p + 1] == 'x' || s[p + 1] == 'X')) {
+        radix = 16;
+        p += 2;
+    } else if (radix == 0) {
+        if (p < length && s[p] == '0')
+            radix = 8;
+        else
+            radix = 10;
+    }
+
+    if (radix < 2 || radix > 36)
+        return NaN;
+
+    bool sawDigit = false;
+    double number = 0;
+    while (p < length) {
+        int digit = parseDigit(s[p].uc, radix);
+        if (digit == -1)
+            break;
+        sawDigit = true;
+        number *= radix;
+        number += digit;
+        ++p;
+    }
+
+    if (!sawDigit)
+        return NaN;
+
+    return sign * number;
+}
+
+static double parseFloat(const UString &s)
+{
+    // Check for 0x prefix here, because toDouble allows it, but we must treat it as 0.
+    // Need to skip any whitespace and then one + or - sign.
+    int length = s.size();
+    int p = 0;
+    while (p < length && isStrWhiteSpace(s[p].uc)) {
+        ++p;
+    }
+    if (p < length && (s[p] == '+' || s[p] == '-')) {
+        ++p;
+    }
+    if (length - p >= 2 && s[p] == '0' && (s[p + 1] == 'x' || s[p + 1] == 'X')) {
+        return 0;
+    }
+
+    return s.toDouble( true /*tolerant*/, false /* NaN for empty string */ );
+}
+
 Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args)
 {
   Value res;
@@ -502,6 +615,7 @@ Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args
     "abcdefghijklmnopqrstuvwxyz"
     "0123456789"
     "*+-./@_";
+
   static const char do_not_escape_when_encoding_URI_component[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
     "abcdefghijklmnopqrstuvwxyz"
@@ -526,7 +640,14 @@ Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args
       int sid;
       int errLine;
       UString errMsg;
-      ProgramNode *progNode = Parser::parse(s.data(),s.size(),&sid,&errLine,&errMsg);
+      SharedPtr<ProgramNode> progNode(Parser::parse(UString(), 0, s.data(),s.size(),&sid,&errLine,&errMsg));
+
+      Debugger *dbg = exec->dynamicInterpreter()->imp()->debugger();
+      if (dbg) {
+        bool cont = dbg->sourceParsed(exec, sid, UString(), s, errLine);
+        if (!cont)
+          return Undefined();
+      }
 
       // no program node means a syntax occurred
       if (!progNode) {
@@ -536,17 +657,15 @@ Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args
         return err;
       }
 
-      progNode->ref();
-
       // enter a new execution context
       Object thisVal(Object::dynamicCast(exec->context().thisValue()));
-      ContextImp ctx(exec->interpreter()->globalObject(),
-                     exec->interpreter()->imp(),
+      ContextImp ctx(exec->dynamicInterpreter()->globalObject(),
+                     exec->dynamicInterpreter()->imp(),
                      thisVal,
                      EvalCode,
                      exec->context().imp());
 
-      ExecState newExec(exec->interpreter(), &ctx);
+      ExecState newExec(exec->dynamicInterpreter(), &ctx);
       newExec.setException(exec->exception()); // could be null
 
       // execute the code
@@ -556,42 +675,19 @@ Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args
       if (newExec.hadException())
         exec->setException(newExec.exception());
 
-      if ( progNode->deref() )
-          delete progNode;
-      if (c.complType() == ReturnValue)
-	  return c.value();
-      // ### setException() on throw?
-      else if (c.complType() == Normal) {
-	  if (c.isValueCompletion())
-	      return c.value();
-	  else
-	      return Undefined();
-      } else {
-	  return Undefined();
-      }
+        res = Undefined();
+        if (c.complType() == Throw)
+            exec->setException(c.value());
+        else if (c.isValueCompletion())
+            res = c.value();
     }
     break;
   }
-  case ParseInt: {
-    CString cstr = args[0].toString(exec).cstring();
-    int radix = args[1].toInt32(exec);
-
-    char* endptr;
-    errno = 0;
-#ifdef HAVE_FUNC_STRTOLL
-    long long llValue = strtoll(cstr.c_str(), &endptr, radix);
-    double value = llValue;
-#else
-    long value = strtol(cstr.c_str(), &endptr, radix);
-#endif
-    if (errno != 0 || endptr == cstr.c_str())
-      res = Number(NaN);
-    else
-      res = Number(value);
+  case ParseInt:
+    res = Number(parseInt(args[0].toString(exec), args[1].toInt32(exec)));
     break;
-  }
   case ParseFloat:
-    res = Number(args[0].toString(exec).toDouble( true /*tolerant*/ ));
+    res = Number(parseFloat(args[0].toString(exec)));
     break;
   case IsNaN:
     res = Boolean(isNaN(args[0].toNumber(exec)));
@@ -614,11 +710,54 @@ Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args
     res = encode(exec, args, do_not_escape_when_encoding_URI_component);
     break;
   case Escape:
-    res = encode(exec, args, do_not_escape);
-    break;
+    {
+      UString r = "", s, str = args[0].toString(exec);
+      const UChar *c = str.data();
+      for (int k = 0; k < str.size(); k++, c++) {
+        int u = c->uc;
+        if (u > 255) {
+          char tmp[7];
+          sprintf(tmp, "%%u%04X", u);
+          s = UString(tmp);
+        } else if (u != 0 && strchr(do_not_escape, (char)u)) {
+          s = UString(c, 1);
+        } else {
+          char tmp[4];
+          sprintf(tmp, "%%%02X", u);
+          s = UString(tmp);
+        }
+        r += s;
+      }
+      res = String(r);
+      break;
+    }
   case UnEscape:
-    res = decode(exec, args, "", false);
-    break;
+    {
+      UString s = "", str = args[0].toString(exec);
+      int k = 0, len = str.size();
+      while (k < len) {
+        const UChar *c = str.data() + k;
+        UChar u;
+        if (*c == UChar('%') && k <= len - 6 && *(c+1) == UChar('u')) {
+          if (Lexer::isHexDigit((c+2)->uc) && Lexer::isHexDigit((c+3)->uc) &&
+              Lexer::isHexDigit((c+4)->uc) && Lexer::isHexDigit((c+5)->uc)) {
+	  u = Lexer::convertUnicode((c+2)->uc, (c+3)->uc,
+				    (c+4)->uc, (c+5)->uc);
+	  c = &u;
+	  k += 5;
+          }
+        } else if (*c == UChar('%') && k <= len - 3 &&
+                   Lexer::isHexDigit((c+1)->uc) && Lexer::isHexDigit((c+2)->uc)) {
+          u = UChar(Lexer::convertHex((c+1)->uc, (c+2)->uc));
+          c = &u;
+          k += 2;
+        }
+        k++;
+        s += UString(c, 1);
+      }
+      res = String(s);
+      break;
+    }
 #ifndef NDEBUG
   case KJSPrint:
     puts(args[0].toString(exec).ascii());
@@ -628,3 +767,5 @@ Value GlobalFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args
 
   return res;
 }
+
+} // namespace
