@@ -76,7 +76,7 @@ void StringInstanceImp::put(ExecState *exec, const Identifier &propertyName, con
   ObjectImp::put(exec, propertyName, value, attr);
 }
 
-bool StringInstanceImp::hasProperty(ExecState *exec, const Identifier &propertyName) const
+bool StringInstanceImp::hasOwnProperty(ExecState *exec, const Identifier &propertyName) const
 {
   if (propertyName == lengthPropertyName)
     return true;
@@ -89,7 +89,7 @@ bool StringInstanceImp::hasProperty(ExecState *exec, const Identifier &propertyN
       return true;
   }
 
-  return ObjectImp::hasProperty(exec, propertyName);
+  return ObjectImp::hasOwnProperty(exec, propertyName);
 }
 
 bool StringInstanceImp::deleteProperty(ExecState *exec, const Identifier &propertyName)
@@ -318,17 +318,16 @@ static Value replace(ExecState *exec, const UString &source, const Value &patter
     delete [] replacements;
 
     return String(result);
-  } else { // First arg is a string
-    UString patternString = pattern.toString(exec);
-    int matchPos = source.find(patternString);
-    int matchLen = patternString.size();
-    // Do the replacement
-    if (matchPos == -1)
-      return String(source);
-    else {
-      return String(source.substr(0, matchPos) + replacement.toString(exec) + source.substr(matchPos + matchLen));
-    }
   }
+  
+  // First arg is a string
+  UString patternString = pattern.toString(exec);
+  int matchPos = source.find(patternString);
+  int matchLen = patternString.size();
+  // Do the replacement
+  if (matchPos == -1)
+    return String(source);
+  return String(source.substr(0, matchPos) + replacement.toString(exec) + source.substr(matchPos + matchLen));
 }
 
 // ECMA 15.5.4.2 - 15.5.4.20
@@ -683,8 +682,7 @@ StringObjectImp::StringObjectImp(ExecState *exec,
   // ECMA 15.5.3.1 String.prototype
   putDirect(prototypePropertyName, stringProto, DontEnum|DontDelete|ReadOnly);
 
-  static Identifier fromCharCode("fromCharCode");
-  putDirect(fromCharCode, new StringObjectFuncImp(exec,funcProto), DontEnum);
+  putDirect(fromCharCodePropertyName, new StringObjectFuncImp(exec, funcProto), DontEnum);
 
   // no. of arguments for constructor
   putDirect(lengthPropertyName, NumberImp::one(), ReadOnly|DontDelete|DontEnum);
